/*
 * Materials for shadowed and or bumped surfaces
 */

#include <common.fxh>

//Bump Specific parameters
float Bumpy = 2.0;

sampler2D normalSampler = sampler_state {
	GenerateMipMap = false;
	MinFilter = LinearMipMapLinear;
	MagFilter = Linear;
	WrapS = Repeat;
	WrapT = Repeat;
	MaxAnisotropy = 16;
};

/************* DATA STRUCTS **************/

struct appdataBump {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
	float3 tangent : ATTR5;
	float3 bitangent : ATTR6;
};

struct vertexOutputBump {
    float4 HPosition	: POSITION;
    float4 TexCoord	: TEXCOORD0; //note: we store HPos2.z in TexCoord.z
	float3 WorldNormal : TEXCOORD1;
    float3 WorldTangent	: TEXCOORD2;
    float3 WorldBitangent	: TEXCOORD3;
	float3 WorldPos : TEXCOORD4;
	float3 WorldEyePos : TEXCOORD5;
	float3 LightVec : TEXCOORD7;
};

/*********** vertex shaders ******/

vertexOutputBump BumpVS(appdataBump IN) {
    
	vertexOutputBump OUT;

	OUT.WorldNormal = mul(WorldIT, IN.Normal).xyz;
	OUT.WorldTangent = mul(WorldIT, float4(IN.tangent,1.0f)).xyz;
	OUT.WorldBitangent = mul(WorldIT, float4(IN.bitangent,1.0f)).xyz;

    float4 Po = float4(IN.Position.xyz,1.0);
    float3 Pw = mul(World, Po).xyz;
    OUT.WorldPos = Pw;
    OUT.TexCoord = IN.UV;
	OUT.WorldEyePos = ViewInv._m03_m13_m23;
    OUT.HPosition = mul(WorldViewProj, Po);

	OUT.LightVec = (LightPos.xyz - Pw);

	OUT.TexCoord.z = mul(WorldViewProj, Po).z;

    return OUT;
}

/********* pixel shaders ********/

float4 BumpPS_4(vertexOutputBump IN, float4 wPos : WPOS) : COLOR {

	float4 colorMap = tex2D(colorSampler,IN.TexCoord.xy);

	#include <ps_bump_code.fxh>
	
	float4 diffContrib = (ldn*LightColor);
	float4 specContrib = ((ldn * hdn) * LightColor);

#ifdef USESHADOWS
	#include <ps_vsdct_code.fxh>

	#include <ps_pcf4tapdither.fxh>
	//#include <ps_pcf16tap.fxh>
#endif 

	#include <ps_result_code.fxh>

	float depth = IN.TexCoord.z;

#ifdef USEDOF
#include <ps_dof_alpha.fxh>
#else
	result.a = 1.0;
#endif

	return result;
}

float4 BumpPS_4_BlurFix(vertexOutputBump IN, float4 wPos : WPOS) : COLOR {

	float4 colorMap = tex2D(colorSampler,IN.TexCoord.xy);

	#include <ps_bump_code.fxh>
	
	float4 diffContrib = (ldn*LightColor);
	float4 specContrib = ((ldn * hdn) * LightColor);
	
#ifdef USESHADOWS
	#include <ps_vsdct_code.fxh>
	#include <ps_pcf4tapdither.fxh>
	//#include <ps_pcf16tap.fxh>
#endif

	float depth = IN.TexCoord.z;

#ifdef USEDOF
#include <ps_result_code.fxh>
#include <ps_dof_alpha.fxh>
	#ifdef USESHADOWS
		#include <ps_vsdctBlurFix.fxh>
	#endif
#else
	#include <ps_result_code.fxh>
	result.a = 1.0;
#endif
//result.x = onEdge;
	return result;
}

technique PerPixel_Shadow4Tap {
	pass p0 {
		#include <commonRenderStates.fxh>
		CullfaceEnable = true;
		CullFace = Back;
		DepthTestEnable = true;
		VertexProgram = compile arbvp1 BumpVS();
		FragmentProgram = compile fp40 BumpPS_4();
	}
}


technique PerPixel_Shadow4Tap_BlurFix {
	pass p0 {
		#include <commonRenderStates.fxh>
		CullfaceEnable = true;
		CullFace = Back;
		DepthTestEnable = true;
		VertexProgram = compile arbvp1 BumpVS();
		FragmentProgram = compile fp40 BumpPS_4_BlurFix();
	}
}


////DEBUG
//float4 texsimple(uniform sampler2D sampler,
//                float2 uv : TEXCOORD0) : COLOR {
//    return tex2D(sampler, uv);
//}
//
//technique TextureOnly {
//    pass {
//		//AlphaTestEnable = false;
//		CullFaceEnable = true;
//		DepthTestEnable = false;
//		//ColorMask =  bool4(false, false, false, false);
//        FragmentProgram = compile arbfp1 texsimple(colorSampler);
//    }
//}
